use domain::{Result, Entity};
use thiserror::Error;
use uuid::Uuid;
use std::marker::PhantomData;
use std::future::Future;

#[derive(Debug, Error)]
pub enum ApplicationError {
    #[error("Domain error: {0}")]
    Domain(#[from] domain::DomainError),
    #[error("Repository error: {0}")]
    Repository(String),
}

pub type AppResult<T> = std::result::Result<T, ApplicationError>;

pub trait Repository<T: Entity>: Send + Sync {
    fn create(&self, data: T::Create) -> impl Future<Output = Result<T>> + Send;
    fn find_by_id(&self, id: Uuid) -> impl Future<Output = Result<T>> + Send;
    fn find_all(&self) -> impl Future<Output = Result<Vec<T>>> + Send;
    fn update(&self, id: Uuid, data: T::Update) -> impl Future<Output = Result<T>> + Send;
    fn delete(&self, id: Uuid) -> impl Future<Output = Result<()>> + Send;
}

pub trait UseCase<T: Entity> {
    fn create(&self, data: T::Create) -> impl Future<Output = AppResult<T>> + Send;
    fn get(&self, id: Uuid) -> impl Future<Output = AppResult<T>> + Send;
    fn list(&self) -> impl Future<Output = AppResult<Vec<T>>> + Send;
    fn update(&self, id: Uuid, data: T::Update) -> impl Future<Output = AppResult<T>> + Send;
    fn delete(&self, id: Uuid) -> impl Future<Output = AppResult<()>> + Send;
}

#[derive(Clone)]
pub struct Service<T: Entity, R: Repository<T> + Clone> {
    repository: R,
    _phantom: PhantomData<T>,
}

impl<T: Entity, R: Repository<T> + Clone> Service<T, R> {
    pub fn new(repository: R) -> Self {
        Self { 
            repository,
            _phantom: PhantomData,
        }
    }
}

impl<T: Entity, R: Repository<T> + Clone> UseCase<T> for Service<T, R> {
    fn create(&self, data: T::Create) -> impl Future<Output = AppResult<T>> + Send {
        async move {
            self.repository.create(data).await.map_err(ApplicationError::Domain)
        }
    }

    fn get(&self, id: Uuid) -> impl Future<Output = AppResult<T>> + Send {
        async move {
            self.repository.find_by_id(id).await.map_err(ApplicationError::Domain)
        }
    }

    fn list(&self) -> impl Future<Output = AppResult<Vec<T>>> + Send {
        async move {
            self.repository.find_all().await.map_err(ApplicationError::Domain)
        }
    }

    fn update(&self, id: Uuid, data: T::Update) -> impl Future<Output = AppResult<T>> + Send {
        async move {
            self.repository.update(id, data).await.map_err(ApplicationError::Domain)
        }
    }

    fn delete(&self, id: Uuid) -> impl Future<Output = AppResult<()>> + Send {
        async move {
            self.repository.delete(id).await.map_err(ApplicationError::Domain)
        }
    }
}
