use std::collections::HashMap;
use std::sync::Arc;
use tokio::sync::RwLock;
use uuid::Uuid;

use domain::{
    CreateProduct, CreateUser, Product, Result, UpdateProduct, UpdateUser, User,
};
use application::{Repository, Service};

#[derive(Default, Clone)]
pub struct InMemoryUserRepository {
    users: Arc<RwLock<HashMap<Uuid, User>>>,
}

impl InMemoryUserRepository {
    pub fn new() -> Self {
        Self {
            users: Arc::new(RwLock::new(HashMap::new())),
        }
    }
}

impl Repository<User> for InMemoryUserRepository {
    async fn create(&self, data: CreateUser) -> Result<User> {
        let user = User {
            id: Uuid::new_v4(),
            username: data.username,
            email: data.email,
            created_at: chrono::Utc::now(),
            updated_at: chrono::Utc::now(),
        };

        self.users.write().await.insert(user.id, user.clone());
        Ok(user)
    }

    async fn find_by_id(&self, id: Uuid) -> Result<User> {
        self.users
            .read()
            .await
            .get(&id)
            .cloned()
            .ok_or_else(|| domain::DomainError::NotFound(format!("User not found: {}", id)))
    }

    async fn find_all(&self) -> Result<Vec<User>> {
        Ok(self.users.read().await.values().cloned().collect())
    }

    async fn update(&self, id: Uuid, data: UpdateUser) -> Result<User> {
        let mut users = self.users.write().await;
        let user = users
            .get_mut(&id)
            .ok_or_else(|| domain::DomainError::NotFound(format!("User not found: {}", id)))?;

        if let Some(username) = data.username {
            user.username = username;
        }
        if let Some(email) = data.email {
            user.email = email;
        }
        user.updated_at = chrono::Utc::now();

        Ok(user.clone())
    }

    async fn delete(&self, id: Uuid) -> Result<()> {
        self.users
            .write()
            .await
            .remove(&id)
            .map(|_| ())
            .ok_or_else(|| domain::DomainError::NotFound(format!("User not found: {}", id)))
    }
}

#[derive(Default, Clone)]
pub struct InMemoryProductRepository {
    products: Arc<RwLock<HashMap<Uuid, Product>>>,
}

impl InMemoryProductRepository {
    pub fn new() -> Self {
        Self {
            products: Arc::new(RwLock::new(HashMap::new())),
        }
    }
}

impl Repository<Product> for InMemoryProductRepository {
    async fn create(&self, data: CreateProduct) -> Result<Product> {
        let product = Product {
            id: Uuid::new_v4(),
            name: data.name,
            description: data.description,
            created_at: chrono::Utc::now(),
            updated_at: chrono::Utc::now(),
        };

        self.products.write().await.insert(product.id, product.clone());
        Ok(product)
    }

    async fn find_by_id(&self, id: Uuid) -> Result<Product> {
        self.products
            .read()
            .await
            .get(&id)
            .cloned()
            .ok_or_else(|| domain::DomainError::NotFound(format!("Product not found: {}", id)))
    }

    async fn find_all(&self) -> Result<Vec<Product>> {
        Ok(self.products.read().await.values().cloned().collect())
    }

    async fn update(&self, id: Uuid, data: UpdateProduct) -> Result<Product> {
        let mut products = self.products.write().await;
        let product = products
            .get_mut(&id)
            .ok_or_else(|| domain::DomainError::NotFound(format!("Product not found: {}", id)))?;

        if let Some(name) = data.name {
            product.name = name;
        }
        if let Some(description) = data.description {
            product.description = description;
        }
        product.updated_at = chrono::Utc::now();

        Ok(product.clone())
    }

    async fn delete(&self, id: Uuid) -> Result<()> {
        self.products
            .write()
            .await
            .remove(&id)
            .map(|_| ())
            .ok_or_else(|| domain::DomainError::NotFound(format!("Product not found: {}", id)))
    }
}

pub fn add(left: u64, right: u64) -> u64 {
    left + right
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn it_works() {
        let result = add(2, 2);
        assert_eq!(result, 4);
    }
}

pub type MemoryUserService = Service<User, InMemoryUserRepository>;
pub type MemoryProductService = Service<Product, InMemoryProductRepository>;
