use sqlx::PgPool;
use uuid::Uuid;

use domain::{
    CreateProduct, CreateUser, Product, Result, UpdateProduct, UpdateUser, User,
};
use application::{Repository, Service};

#[derive(Clone)]
pub struct PostgresUserRepository {
    pool: PgPool,
}

impl PostgresUserRepository {
    pub fn new(pool: PgPool) -> Self {
        Self { pool }
    }
}

impl Repository<User> for PostgresUserRepository {
    async fn create(&self, data: CreateUser) -> Result<User> {
        let user = sqlx::query_as!(
            User,
            r#"
            INSERT INTO users (username, email)
            VALUES ($1, $2)
            RETURNING id, username, email, created_at, updated_at
            "#,
            data.username,
            data.email
        )
        .fetch_one(&self.pool)
        .await
        .map_err(|e| domain::DomainError::Internal(e.to_string()))?;

        Ok(user)
    }

    async fn find_by_id(&self, id: Uuid) -> Result<User> {
        let user = sqlx::query_as!(
            User,
            r#"
            SELECT id, username, email, created_at, updated_at
            FROM users
            WHERE id = $1
            "#,
            id
        )
        .fetch_optional(&self.pool)
        .await
        .map_err(|e| domain::DomainError::Internal(e.to_string()))?
        .ok_or_else(|| domain::DomainError::NotFound(format!("User not found: {}", id)))?;

        Ok(user)
    }

    async fn find_all(&self) -> Result<Vec<User>> {
        let users = sqlx::query_as!(
            User,
            r#"
            SELECT id, username, email, created_at, updated_at
            FROM users
            "#
        )
        .fetch_all(&self.pool)
        .await
        .map_err(|e| domain::DomainError::Internal(e.to_string()))?;

        Ok(users)
    }

    async fn update(&self, id: Uuid, data: UpdateUser) -> Result<User> {
        let user = sqlx::query_as!(
            User,
            r#"
            UPDATE users
            SET
                username = COALESCE($1, username),
                email = COALESCE($2, email),
                updated_at = CURRENT_TIMESTAMP
            WHERE id = $3
            RETURNING id, username, email, created_at, updated_at
            "#,
            data.username,
            data.email,
            id
        )
        .fetch_optional(&self.pool)
        .await
        .map_err(|e| domain::DomainError::Internal(e.to_string()))?
        .ok_or_else(|| domain::DomainError::NotFound(format!("User not found: {}", id)))?;

        Ok(user)
    }

    async fn delete(&self, id: Uuid) -> Result<()> {
        let result = sqlx::query!(
            r#"
            DELETE FROM users
            WHERE id = $1
            "#,
            id
        )
        .execute(&self.pool)
        .await
        .map_err(|e| domain::DomainError::Internal(e.to_string()))?;

        if result.rows_affected() == 0 {
            return Err(domain::DomainError::NotFound(format!("User not found: {}", id)));
        }

        Ok(())
    }
}

#[derive(Clone)]
pub struct PostgresProductRepository {
    pool: PgPool,
}

impl PostgresProductRepository {
    pub fn new(pool: PgPool) -> Self {
        Self { pool }
    }
}

impl Repository<Product> for PostgresProductRepository {
    async fn create(&self, data: CreateProduct) -> Result<Product> {
        let product = sqlx::query_as!(
            Product,
            r#"
            INSERT INTO products (name, description)
            VALUES ($1, $2)
            RETURNING id, name, description, created_at, updated_at
            "#,
            data.name,
            data.description
        )
        .fetch_one(&self.pool)
        .await
        .map_err(|e| domain::DomainError::Internal(e.to_string()))?;

        Ok(product)
    }

    async fn find_by_id(&self, id: Uuid) -> Result<Product> {
        let product = sqlx::query_as!(
            Product,
            r#"
            SELECT id, name, description, created_at, updated_at
            FROM products
            WHERE id = $1
            "#,
            id
        )
        .fetch_optional(&self.pool)
        .await
        .map_err(|e| domain::DomainError::Internal(e.to_string()))?
        .ok_or_else(|| domain::DomainError::NotFound(format!("Product not found: {}", id)))?;

        Ok(product)
    }

    async fn find_all(&self) -> Result<Vec<Product>> {
        let products = sqlx::query_as!(
            Product,
            r#"
            SELECT id, name, description, created_at, updated_at
            FROM products
            "#
        )
        .fetch_all(&self.pool)
        .await
        .map_err(|e| domain::DomainError::Internal(e.to_string()))?;

        Ok(products)
    }

    async fn update(&self, id: Uuid, data: UpdateProduct) -> Result<Product> {
        let product = sqlx::query_as!(
            Product,
            r#"
            UPDATE products
            SET
                name = COALESCE($1, name),
                description = COALESCE($2, description),
                updated_at = CURRENT_TIMESTAMP
            WHERE id = $3
            RETURNING id, name, description, created_at, updated_at
            "#,
            data.name,
            data.description,
            id
        )
        .fetch_optional(&self.pool)
        .await
        .map_err(|e| domain::DomainError::Internal(e.to_string()))?
        .ok_or_else(|| domain::DomainError::NotFound(format!("Product not found: {}", id)))?;

        Ok(product)
    }

    async fn delete(&self, id: Uuid) -> Result<()> {
        let result = sqlx::query!(
            r#"
            DELETE FROM products
            WHERE id = $1
            "#,
            id
        )
        .execute(&self.pool)
        .await
        .map_err(|e| domain::DomainError::Internal(e.to_string()))?;

        if result.rows_affected() == 0 {
            return Err(domain::DomainError::NotFound(format!("Product not found: {}", id)));
        }

        Ok(())
    }
}

pub type PostgresUserService = Service<User, PostgresUserRepository>;
pub type PostgresProductService = Service<Product, PostgresProductRepository>;
